import codecommit = require('@aws-cdk/aws-codecommit');
import s3 = require('@aws-cdk/aws-s3');
import { SecretValue } from '@aws-cdk/cdk';
import { CfnProject } from './codebuild.generated';
/**
 * Properties common to all Source classes.
 */
export interface BuildSourceProps {
    /**
     * The source identifier.
     * This property is required on secondary sources.
     */
    readonly identifier?: string;
}
/**
 * Source provider definition for a CodeBuild Project.
 */
export declare abstract class BuildSource {
    readonly identifier?: string;
    abstract readonly type: SourceType;
    constructor(props: BuildSourceProps);
    toSourceJSON(): CfnProject.SourceProperty;
    buildTriggers(): CfnProject.ProjectTriggersProperty | undefined;
    protected toSourceProperty(): any;
}
/**
 * A `NO_SOURCE` CodeBuild Project Source definition.
 * This is the default source type,
 * if none was specified when creating the Project.
 * *Note*: the `NO_SOURCE` type cannot be used as a secondary source,
 * and because of that, you're not allowed to specify an identifier for it.
 */
export declare class NoSource extends BuildSource {
    readonly type: SourceType;
    constructor();
}
/**
 * The construction properties common to all build sources that are backed by Git.
 */
export interface GitBuildSourceProps extends BuildSourceProps {
    /**
     * The depth of history to download. Minimum value is 0.
     * If this value is 0, greater than 25, or not provided,
     * then the full history is downloaded with each build of the project.
     */
    readonly cloneDepth?: number;
}
/**
 * A common superclass of all build sources that are backed by Git.
 */
export declare abstract class GitBuildSource extends BuildSource {
    private readonly cloneDepth?;
    protected constructor(props: GitBuildSourceProps);
    toSourceJSON(): CfnProject.SourceProperty;
}
/**
 * Construction properties for {@link CodeCommitSource}.
 */
export interface CodeCommitSourceProps extends GitBuildSourceProps {
    readonly repository: codecommit.IRepository;
}
/**
 * CodeCommit Source definition for a CodeBuild project.
 */
export declare class CodeCommitSource extends GitBuildSource {
    readonly type: SourceType;
    private readonly repo;
    constructor(props: CodeCommitSourceProps);
    protected toSourceProperty(): any;
}
/**
 * Construction properties for {@link S3BucketSource}.
 */
export interface S3BucketSourceProps extends BuildSourceProps {
    readonly bucket: s3.IBucket;
    readonly path: string;
}
/**
 * S3 bucket definition for a CodeBuild project.
 */
export declare class S3BucketSource extends BuildSource {
    readonly type: SourceType;
    private readonly bucket;
    private readonly path;
    constructor(props: S3BucketSourceProps);
    protected toSourceProperty(): any;
}
/**
 * CodePipeline Source definition for a CodeBuild Project.
 * *Note*: this type cannot be used as a secondary source,
 * and because of that, you're not allowed to specify an identifier for it.
 */
export declare class CodePipelineSource extends BuildSource {
    readonly type: SourceType;
    constructor();
}
/**
 * Construction properties for {@link GitHubSource} and {@link GitHubEnterpriseSource}.
 */
export interface GitHubSourceProps extends GitBuildSourceProps {
    /**
     * The GitHub account/user that owns the repo.
     *
     * @example 'awslabs'
     */
    readonly owner: string;
    /**
     * The name of the repo (without the username).
     *
     * @example 'aws-cdk'
     */
    readonly repo: string;
    /**
     * The oAuthToken used to authenticate when cloning source git repo.
     * Note that you need to give CodeBuild permissions to your GitHub account in order for the token to work.
     * That is a one-time operation that can be done through the AWS Console for CodeBuild.
     */
    readonly oauthToken: SecretValue;
    /**
     * Whether to create a webhook that will trigger a build every time a commit is pushed to the GitHub repository.
     *
     * @default false
     */
    readonly webhook?: boolean;
    /**
     * Whether to send GitHub notifications on your build's start and end.
     *
     * @default true
     */
    readonly reportBuildStatus?: boolean;
}
/**
 * GitHub Source definition for a CodeBuild project.
 */
export declare class GitHubSource extends GitBuildSource {
    readonly type: SourceType;
    private readonly httpsCloneUrl;
    private readonly oauthToken;
    private readonly reportBuildStatus;
    private readonly webhook?;
    constructor(props: GitHubSourceProps);
    buildTriggers(): CfnProject.ProjectTriggersProperty | undefined;
    protected toSourceProperty(): any;
}
/**
 * Construction properties for {@link GitHubEnterpriseSource}.
 */
export interface GitHubEnterpriseSourceProps extends GitBuildSourceProps {
    /**
     * The HTTPS URL of the repository in your GitHub Enterprise installation.
     */
    readonly httpsCloneUrl: string;
    /**
     * The OAuth token used to authenticate when cloning the git repository.
     */
    readonly oauthToken: SecretValue;
    /**
     * Whether to ignore SSL errors when connecting to the repository.
     *
     * @default false
     */
    readonly ignoreSslErrors?: boolean;
}
/**
 * GitHub Enterprise Source definition for a CodeBuild project.
 */
export declare class GitHubEnterpriseSource extends GitBuildSource {
    readonly type: SourceType;
    private readonly httpsCloneUrl;
    private readonly oauthToken;
    private readonly ignoreSslErrors?;
    constructor(props: GitHubEnterpriseSourceProps);
    protected toSourceProperty(): any;
}
/**
 * Construction properties for {@link BitBucketSource}.
 */
export interface BitBucketSourceProps extends GitBuildSourceProps {
    /**
     * The BitBucket account/user that owns the repo.
     *
     * @example 'awslabs'
     */
    readonly owner: string;
    /**
     * The name of the repo (without the username).
     *
     * @example 'aws-cdk'
     */
    readonly repo: string;
}
/**
 * BitBucket Source definition for a CodeBuild project.
 */
export declare class BitBucketSource extends GitBuildSource {
    readonly type: SourceType;
    private readonly httpsCloneUrl;
    constructor(props: BitBucketSourceProps);
    protected toSourceProperty(): any;
}
/**
 * Source types for CodeBuild Project
 */
export declare enum SourceType {
    None = "NO_SOURCE",
    CodeCommit = "CODECOMMIT",
    CodePipeline = "CODEPIPELINE",
    GitHub = "GITHUB",
    GitHubEnterprise = "GITHUB_ENTERPRISE",
    BitBucket = "BITBUCKET",
    S3 = "S3"
}
